@@ -1,3 +1,6 @@
+// -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
+// vim: ts=8 sw=2 smarttab
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -54,7 +57,6 @@
 #include "rgw_resolve.h"
 #include "rgw_loadgen.h"
 #include "rgw_civetweb.h"
-#include "rgw_civetweb_log.h"
 
 #include "civetweb/civetweb.h"
 
@@ -91,11 +93,9 @@ struct RGWRequest
   RGWOp *op;
   utime_t ts;
 
-  RGWRequest(uint64_t id) : id(id), s(NULL), op(NULL) {
+  RGWRequest() : id(0), s(NULL), op(NULL) {
   }
 
-  virtual ~RGWRequest() {}
-
   void init_state(req_state *_s) {
     s = _s;
   }
@@ -144,17 +144,12 @@ public:
   bool get_val(const string& key, const string& def_val, string *out);
   bool get_val(const string& key, int def_val, int *out);
 
-  map<string, string>& get_config_map() { return config_map; }
-
   string get_framework() { return framework; }
 };
 
 
 struct RGWFCGXRequest : public RGWRequest {
   FCGX_Request fcgx;
-
-  RGWFCGXRequest(uint64_t req_id) : RGWRequest(req_id) {}
-
 };
 
 struct RGWProcessEnv {
@@ -225,6 +220,8 @@ protected:
     }
   } req_wq;
 
+  uint64_t max_req_id;
+
 public:
   RGWProcess(CephContext *cct, RGWProcessEnv *pe, int num_threads, RGWFrontendConfig *_conf)
     : store(pe->store), olog(pe->olog), m_tp(cct, "RGWProcess::m_tp", num_threads),
@@ -233,7 +230,8 @@ public:
       conf(_conf),
       sock_fd(-1),
       req_wq(this, g_conf->rgw_op_thread_timeout,
-	     g_conf->rgw_op_thread_suicide_timeout, &m_tp) {}
+	     g_conf->rgw_op_thread_suicide_timeout, &m_tp),
+      max_req_id(0) {}
   virtual ~RGWProcess() {}
   virtual void run() = 0;
   virtual void handle_request(RGWRequest *req) = 0;
@@ -261,17 +259,10 @@ void RGWFCGXProcess::run()
   string socket_port;
   string socket_host;
 
-  conf->get_val("socket_path", "", &socket_path);
+  conf->get_val("socket_path", g_conf->rgw_socket_path, &socket_path);
   conf->get_val("socket_port", g_conf->rgw_port, &socket_port);
   conf->get_val("socket_host", g_conf->rgw_host, &socket_host);
 
-  if (socket_path.empty() && socket_port.empty() && socket_host.empty()) {
-    socket_path = g_conf->rgw_socket_path;
-    if (socket_path.empty()) {
-      dout(0) << "ERROR: no socket server point defined, cannot start fcgi frontend" << dendl;
-      return;
-    }
-  }
 
   if (!socket_path.empty()) {
     string path_str = socket_path;
@@ -310,7 +301,8 @@ void RGWFCGXProcess::run()
   m_tp.start();
 
   for (;;) {
-    RGWFCGXRequest *req = new RGWFCGXRequest(store->get_new_req_id());
+    RGWFCGXRequest *req = new RGWFCGXRequest;
+    req->id = ++max_req_id;
     dout(10) << "allocated request req=" << hex << req << dec << dendl;
     FCGX_InitRequest(&req->fcgx, sock_fd, 0);
     req_throttle.get(1);
@@ -336,8 +328,8 @@ struct RGWLoadGenRequest : public RGWRequest {
   atomic_t *fail_flag;
 
 
-  RGWLoadGenRequest(uint64_t req_id, const string& _m, const  string& _r, int _cl,
-                    atomic_t *ff) : RGWRequest(req_id), method(_m), resource(_r), content_length(_cl), fail_flag(ff) {}
+  RGWLoadGenRequest(const string& _m, const  string& _r, int _cl,
+                    atomic_t *ff) : method(_m), resource(_r), content_length(_cl), fail_flag(ff) {}
 };
 
 class RGWLoadGenProcess : public RGWProcess {
@@ -371,7 +363,7 @@ void RGWLoadGenProcess::run()
   int num_buckets;
   conf->get_val("num_buckets", 1, &num_buckets);
 
-  string buckets[num_buckets];
+  vector<string> buckets(num_buckets);
 
   atomic_t failed;
 
@@ -438,8 +430,8 @@ done:
 
 void RGWLoadGenProcess::gen_request(const string& method, const string& resource, int content_length, atomic_t *fail_flag)
 {
-  RGWLoadGenRequest *req = new RGWLoadGenRequest(store->get_new_req_id(), method, resource,
-						 content_length, fail_flag);
+  RGWLoadGenRequest *req = new RGWLoadGenRequest(method, resource, content_length, fail_flag);
+  req->id = ++max_req_id;
   dout(10) << "allocated request req=" << hex << req << dec << dendl;
   req_throttle.get(1);
   req_wq.queue(req);
@@ -637,7 +629,7 @@ void RGWFCGXProcess::handle_request(RGWRequest *r)
   RGWFCGXRequest *req = static_cast<RGWFCGXRequest *>(r);
   FCGX_Request *fcgx = &req->fcgx;
   RGWFCGX client_io(fcgx);
-  
+
  
   int ret = process_request(store, rest, req, &client_io, olog);
   if (ret < 0) {
@@ -647,10 +639,6 @@ void RGWFCGXProcess::handle_request(RGWRequest *r)
 
   FCGX_Finish_r(fcgx);
 
-  if (store->ctx()->_conf->rgw_fcgi_explicit_free) {
-    FCGX_Free(fcgx, 1);
-  }
-
   delete req;
 }
 
@@ -688,12 +676,12 @@ void RGWLoadGenProcess::handle_request(RGWRequest *r)
 
 static int civetweb_callback(struct mg_connection *conn) {
   struct mg_request_info *req_info = mg_get_request_info(conn);
-  RGWProcessEnv *pe = (RGWProcessEnv *)req_info->user_data;
+  RGWProcessEnv *pe = static_cast<RGWProcessEnv *>(req_info->user_data);
   RGWRados *store = pe->store;
   RGWREST *rest = pe->rest;
   OpsLogSocket *olog = pe->olog;
 
-  RGWRequest *req = new RGWRequest(store->get_new_req_id());
+  RGWRequest *req = new RGWRequest;
   RGWMongoose client_io(conn, pe->port);
 
   client_io.init(g_ceph_context);
@@ -736,9 +724,15 @@ int usage()
 {
   cerr << "usage: radosgw [options...]" << std::endl;
   cerr << "options:\n";
-  cerr << "   --rgw-region=<region>     region in which radosgw runs\n";
-  cerr << "   --rgw-zone=<zone>         zone in which radosgw runs\n";
+  cerr << "  --rgw-region=<region>     region in which radosgw runs\n";
+  cerr << "  --rgw-zone=<zone>         zone in which radosgw runs\n";
+  cerr << "  --rgw-socket-path=<path>  specify a unix domain socket path\n";
   generic_server_usage();
+  cerr << "  -m monaddress[:port]      connect to specified monitor\n";
+  cerr << "  --keyring=<path>          path to radosgw keyring\n";
+  cerr << "  --logfile=<logfile>       file to log debug output\n";
+  cerr << "  --debug-rgw=<log-level>/<memory-level>  set radosgw debug level\n";
+
   return 0;
 }
 
@@ -836,7 +830,7 @@ public:
   void *entry() {
     pprocess->run();
     return NULL;
-  };
+  }
 };
 
 class RGWProcessFrontend : public RGWFrontend {
@@ -924,12 +918,6 @@ class RGWMongooseFrontend : public RGWFrontend {
   struct mg_context *ctx;
   RGWProcessEnv env;
 
-  void set_conf_default(map<string, string>& m, const string& key, const string& def_val) {
-    if (m.find(key) == m.end()) {
-      m[key] = def_val;
-    }
-  }
-
 public:
   RGWMongooseFrontend(RGWProcessEnv& pe, RGWFrontendConfig *_conf) : conf(_conf), ctx(NULL), env(pe) {
   }
@@ -942,28 +930,12 @@ public:
     char thread_pool_buf[32];
     snprintf(thread_pool_buf, sizeof(thread_pool_buf), "%d", (int)g_conf->rgw_thread_pool_size);
     string port_str;
-    map<string, string> conf_map = conf->get_config_map();
     conf->get_val("port", "80", &port_str);
-    conf_map.erase("port");
-    conf_map["listening_ports"] = port_str;
-    set_conf_default(conf_map, "enable_keep_alive", "yes");
-    set_conf_default(conf_map, "num_threads", thread_pool_buf);
-    set_conf_default(conf_map, "decode_url", "no");
-
-    const char *options[conf_map.size() * 2 + 1];
-    int i = 0;
-    for (map<string, string>::iterator iter = conf_map.begin(); iter != conf_map.end(); ++iter) {
-      options[i] = iter->first.c_str();
-      options[i + 1] = iter->second.c_str();
-      dout(20)<< "civetweb config: " << options[i] << ": " << (options[i + 1] ? options[i + 1] : "<null>") << dendl;
-      i += 2;
-    }
-    options[i] = NULL;
+    const char *options[] = {"listening_ports", port_str.c_str(), "enable_keep_alive", "yes", "num_threads", thread_pool_buf, NULL};
 
     struct mg_callbacks cb;
     memset((void *)&cb, 0, sizeof(cb));
     cb.begin_request = civetweb_callback;
-    cb.log_message = rgw_civetweb_log_callback;
     ctx = mg_start(&cb, &env, (const char **)&options);
 
     if (!ctx) {
@@ -1002,7 +974,7 @@ int main(int argc, const char **argv)
   vector<const char *> def_args;
   def_args.push_back("--debug-rgw=1/5");
   def_args.push_back("--keyring=$rgw_data/keyring");
-  def_args.push_back("--log-file=/var/log/radosgw/$cluster-$name.log");
+  def_args.push_back("--log-file=/var/log/radosgw/$cluster-$name");
 
   vector<const char*> args;
   argv_to_vec(argc, argv, args);
@@ -1120,10 +1092,7 @@ int main(int argc, const char **argv)
   register_async_signal_handler(SIGUSR1, handle_sigterm);
   sighandler_alrm = signal(SIGALRM, godown_alarm);
 
-  string frontend_frameworks = g_conf->rgw_frontends;
-
   list<string> frontends;
-
   get_str_list(g_conf->rgw_frontends, ",", frontends);
 
   multimap<string, RGWFrontendConfig *> fe_map;
@@ -1144,7 +1113,7 @@ int main(int argc, const char **argv)
     configs.push_back(config);
 
     string framework = config->get_framework();
-    fe_map.insert(make_pair<string, RGWFrontendConfig *>(framework, config));
+    fe_map.insert(pair<string, RGWFrontendConfig*>(framework, config));
   }
 
   list<RGWFrontend *> fes;
@@ -1158,8 +1127,6 @@ int main(int argc, const char **argv)
 
       fe = new RGWFCGXFrontend(fcgi_pe, config);
     } else if (framework == "civetweb" || framework == "mongoose") {
-      string err;
-
       int port;
       config->get_val("port", 80, &port);
 
