@@ -1,3 +1,6 @@
+// -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
+// vim: ts=8 sw=2 smarttab
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -505,8 +508,6 @@ int RGWSwift::validate_keystone_token(RGWRados *store, const string& token, stru
 
     validate.append_header("X-Auth-Token", admin_token);
 
-    validate.set_send_length(0);
-
     int ret = validate.process(url.c_str());
     if (ret < 0)
       return ret;
@@ -520,8 +521,6 @@ int RGWSwift::validate_keystone_token(RGWRados *store, const string& token, stru
   if (ret < 0)
     return ret;
 
-  keystone_token_cache->add(token_id, t);
-
   ret = update_user_info(store, info, rgw_user);
   if (ret < 0)
     return ret;
@@ -612,34 +611,6 @@ int authenticate_temp_url(RGWRados *store, req_state *s)
 }
 
 bool RGWSwift::verify_swift_token(RGWRados *store, req_state *s)
-{
-  if (!do_verify_swift_token(store, s)) {
-    return false;
-  }
-
-  if (!s->swift_user.empty()) {
-    string subuser;
-    ssize_t pos = s->swift_user.find(':');
-    if (pos < 0) {
-      subuser = s->swift_user;
-    } else {
-      subuser = s->swift_user.substr(pos + 1);
-    }
-    s->perm_mask = 0;
-    map<string, RGWSubUser>::iterator iter = s->user.subusers.find(subuser);
-    if (iter != s->user.subusers.end()) {
-      RGWSubUser& subuser = iter->second;
-      s->perm_mask = subuser.perm_mask;
-    }
-  } else {
-    s->perm_mask = RGW_PERM_FULL_CONTROL;
-  }
-
-  return true;
-
-}
-
-bool RGWSwift::do_verify_swift_token(RGWRados *store, req_state *s)
 {
   if (!s->os_auth_token) {
     int ret = authenticate_temp_url(store, s);
@@ -647,7 +618,7 @@ bool RGWSwift::do_verify_swift_token(RGWRados *store, req_state *s)
   }
 
   if (strncmp(s->os_auth_token, "AUTH_rgwtk", 10) == 0) {
-    int ret = rgw_swift_verify_signed_token(s->cct, store, s->os_auth_token, s->user, &s->swift_user);
+    int ret = rgw_swift_verify_signed_token(s->cct, store, s->os_auth_token, s->user);
     if (ret < 0)
       return false;
 
