@@ -21,6 +21,7 @@ export PATH=:$PATH # make sure program from sources are prefered
 DIR=test-ceph-disk
 MON_ID=a
 MONA=127.0.0.1:7451
+TEST_POOL=rbd
 FSID=$(uuidgen)
 export CEPH_CONF=/dev/null
 export CEPH_ARGS="--fsid $FSID"
@@ -35,10 +36,8 @@ CEPH_DISK_ARGS+=" --statedir=$DIR"
 CEPH_DISK_ARGS+=" --sysconfdir=$DIR"
 CEPH_DISK_ARGS+=" --prepend-to-path="
 CEPH_DISK_ARGS+=" --verbose"
-TIMEOUT=3600
+TIMEOUT=360
 
-mkdir=$(which mkdir)
-touch=$(which touch)
 cat=$(which cat)
 timeout=$(which timeout)
 diff=$(which diff)
@@ -56,15 +55,14 @@ function teardown() {
 
 function run_mon() {
     local mon_dir=$DIR/$MON_ID
-    $mkdir -p $mon_dir
+
     ./ceph-mon \
         --id $MON_ID \
         --mkfs \
         --mon-data=$mon_dir \
-        --keyring=/dev/null \
         --mon-initial-members=$MON_ID \
         "$@"
-    $touch $mon_dir/keyring
+
     ./ceph-mon \
         --id $MON_ID \
         --mon-data=$mon_dir \
@@ -75,8 +73,9 @@ function run_mon() {
 
 function kill_daemons() {
     for pidfile in $(find $DIR | grep pidfile) ; do
+        pid=$(cat $pidfile)
         for try in 0 1 1 1 2 3 ; do
-            kill $(cat $pidfile) || break
+            kill $pid || break
             sleep $try
         done
     done
@@ -85,7 +84,7 @@ function kill_daemons() {
 function command_fixture() {
     local command=$1
 
-    [ $(which $command) = ./$command ] || return 1
+    [ $(which $command) = ./$command ] || [ $(which $command) = $(pwd)/$command ] || return 1
 
     cat > $DIR/$command <<EOF
 #!/bin/bash
@@ -170,8 +169,7 @@ function test_activate_dir_magic() {
     CEPH_ARGS="--fsid $uuid" \
      ./ceph-disk $CEPH_DISK_ARGS prepare $osd_data 2>&1 | tee $DIR/out
     grep --quiet 'Preparing osd data dir' $DIR/out || return 1
-# does not work because of CEPH_ARGS parsing precendence that was fixed post emperor
-#    grep --quiet $uuid $osd_data/ceph_fsid || return 1
+    grep --quiet $uuid $osd_data/ceph_fsid || return 1
     [ -f $osd_data/magic ] || return 1
 
     echo will not override an existing OSD
@@ -179,7 +177,7 @@ function test_activate_dir_magic() {
     CEPH_ARGS="--fsid $(uuidgen)" \
      ./ceph-disk $CEPH_DISK_ARGS prepare $osd_data 2>&1 | tee $DIR/out
     grep --quiet 'ceph-disk:Data dir .* already exists' $DIR/out || return 1
-#    grep --quiet $uuid $osd_data/ceph_fsid || return 1
+    grep --quiet $uuid $osd_data/ceph_fsid || return 1
 }
 
 function test_activate_dir() {
@@ -195,13 +193,13 @@ function test_activate_dir() {
                       activate \
                      --mark-init=none \
                     $osd_data || return 1
-    $timeout $TIMEOUT ./ceph osd pool set data size 1 || return 1
+    $timeout $TIMEOUT ./ceph osd pool set $TEST_POOL size 1 || return 1
     local id=$($cat $osd_data/whoami)
     local weight=1
     ./ceph osd crush add osd.$id $weight root=default host=localhost || return 1
     echo FOO > $DIR/BAR
-    $timeout $TIMEOUT ./rados --pool data put BAR $DIR/BAR || return 1
-    $timeout $TIMEOUT ./rados --pool data get BAR $DIR/BAR.copy || return 1
+    $timeout $TIMEOUT ./rados --pool $TEST_POOL put BAR $DIR/BAR || return 1
+    $timeout $TIMEOUT ./rados --pool $TEST_POOL get BAR $DIR/BAR.copy || return 1
     $diff $DIR/BAR $DIR/BAR.copy || return 1
 }
 
